class AbortPromise {
  lock = false;

  constructor(fn) {
    const promise = new Promise( (resolve, reject) => {
      this.abort = this.createAbort(reject);
      this.applyGenerator(fn(resolve, reject));
    });
    promise.abort = this.abort;
    promise.catch(() => {});
    return promise;
  }

  // 执行 generator 方法
  applyGenerator(generator, data) {
    if (this.lock) {
      return;
    }
    const { value, done } = generator.next(data);
    if (done) {
      return value;
    }
    if (Object.prototype.toString.call(value) === '[object Promise]') {
      value.then(result => {
        this.applyGenerator(generator, result);
      }).catch((result) => {
        generator.throw(result);
      });
    } else {
      this.applyGenerator(generator, value);
    }
  }

  // 创建取消方法
  createAbort(reject) {
    const that = this;
    return function () {
      that.lock = true;
      reject(...arguments);
    }
  }

}

export default AbortPromise;
